ArgMorph subclass: #ChoiceArgMorph
	instanceVariableNames: 'menuIconMorph labelIconMorph isBoolean options choice getOptionsSelector '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-Blocks'!
!ChoiceArgMorph commentStamp: '<historical>' prior: 0!
I am used to represent an ArgMorph that allows the user to choose from a popup menu of choices. For example, I might allow selection of a sensor port from the list ('a', 'b', 'c', 'd').

In an earlier version of Scratch, I was used to represent boolean values. That usage has been replaced with BooleanArgMorph.
!


!ChoiceArgMorph methodsFor: 'initialization' stamp: 'CWB 11/24/2012 15:53'!
initialize

	super initialize.
	self borderInset.
	self borderWidth: 1.
	self color: Color brown.
	labelMorph _ StringMorph new
		font: (ScratchFrameMorph getFont: #Arg);
		forceUnicodeRendering: true;
		color: Color white.
	self addMorph: labelMorph.
	labelIconMorph _ TranslucentImageMorph new.
	labelIconMorph isHidden: true.
	self addMorph: labelIconMorph.
	menuIconMorph _ ImageMorph new form: DropDownMenuForm.
	self addMorphFront: menuIconMorph.
	isBoolean _ false.
	options _ #(a b c d).
	self choice: #a.
! !


!ChoiceArgMorph methodsFor: 'accessing' stamp: 'CWB 12/17/2010 11:22'!
choice
"Note that if the list was translated, the choice is still returned in English."

	^ choice
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'CWB 11/24/2012 15:01'!
choice: aSymbol
	"Update the toggle button when an argument is changed within a block."

	| frame palette block doLocalization label contentsMorph |
	frame _ self ownerThatIsA: ScratchFrameMorph.
	((#soundNames = getOptionsSelector) and:
	 [aSymbol = ('record' localized, ScratchTranslator ellipsesSuffix)]) ifTrue: [
		frame ifNotNil: [^ frame newSound]].

	((#sceneNames = getOptionsSelector) and:
	 [aSymbol = ('record' localized, ScratchTranslator ellipsesSuffix)]) ifTrue: [
		frame ifNotNil: [^ frame workPane newScene]].

	((#sceneNames = getOptionsSelector) and:
	 [aSymbol = ('re-record' localized, ScratchTranslator ellipsesSuffix)]) ifTrue: [
		frame ifNotNil: [^ frame workPane rerecordScene]].

	choice _ label _ aSymbol.
	label isUnicode ifFalse: [label _ label asString].
	doLocalization _ (#(costumeNames soundNames varNamesMenu listVarMenu) includes: getOptionsSelector) not. 
	doLocalization & label isUnicode not
		ifTrue:[labelMorph contents: label localized]
		ifFalse:[labelMorph contents: label].
	options = {'with inputs' asUTF8 localized. 'with input list' asUTF8 localized} ifTrue: [
		labelMorph contents: label asMacRoman localized].
	labelMorph position: self position + borderWidth + (3 @ -1).
	labelIconMorph position: self position + borderWidth + (3 @ 0).

	"Should we show an image instead of a textual label?"
	contentsMorph _ labelMorph.
	(aSymbol asString beginsWith: 'img:') ifTrue: [
		| imageForm imageName |

		imageName _ (aSymbol asString allButFirst: 4).
		imageForm _ ScratchFrameMorph 
			skinAt: (imageName asSymbol) 
			ifAbsent: [ imageForm _ nil ].

		imageForm ifNotNil: [
			labelIconMorph form: imageForm.
			contentsMorph _ labelIconMorph.
		].
	].
	labelIconMorph isHidden: (contentsMorph ~= labelIconMorph).
	labelMorph isHidden: (contentsMorph ~= labelMorph).

	self extent: (contentsMorph extent + (16@2) + borderWidth).

	menuIconMorph position: (self right - borderWidth - 10)@(self top + borderWidth + 4).

	(owner isKindOf: BlockMorph) ifTrue: [
		((owner respondsTo: #addLabel) and: [owner isCustomBlock not])
			ifTrue: [owner addLabel]
			ifFalse: [owner fixBlockLayout]].

	(owner isKindOf: SetterBlockMorph) ifTrue: [ owner determineIfSpecialForm ].

	options = {'with inputs' asUTF8 localized. 'with input list' asUTF8 localized} ifTrue: [
		(owner respondsTo: #adjustInputParadigm:) ifTrue: [
			owner adjustInputParadigm: choice asMacRoman localized]]. 

	(palette _ self ownerThatIsA: ScratchBlockPaletteMorph) ifNil: [^ self].
	(block _ self ownerThatIsA: BlockMorph) ifNil: [^ self].

	block isReporter ifTrue: [palette updateWatcherButtonsForFrame: frame].
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'jens 2/3/2010 00:17'!
defaultValue: aString
	"Set my default value."

	self choice: aString.
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'CWB 9/10/2012 06:59'!
exportArgWith: anExporter havingSpec: aSpec
	"Exports the given argument"

	| english type fStream |	

	english _ self choice.  "self labelMorph contents."

	"add prefix to words command / predicate / reporter to avoid clash with exporting custom blocks"
	((english = 'command') | (english = 'reporter') | (english = 'predicate')) ifTrue: [
		english _ 'isa' , english.
	].

	"What sort of variable are we dealing with?"
	type _ aSpec second.	"%n2 -> $n, a number type"

	"Variable and list names should be replaced with safe ascii representations"
	(type = $v) | (type = $L) ifTrue: 
		[^ anExporter codeStream nextPutAll: (anExporter shortNameFor: english)].

	"Remote sensors just go out as english"
	(type = $H) ifTrue: 
		[^ anExporter codeStream nextPutAll: '"', english, '"'].

	"Costume names go out as strings."
	(type = $l) ifTrue:
		[^ anExporter exportLiteralValue: english on: anExporter codeStream ].

	(type = $S) ifTrue: 
		[
			"Write the filename for the sound in the program script to a file for uploader to use when uploading media assets to the NXT."
			fStream := CrLfFileStream fileNamed: ((ScratchFileChooserDialog userEnchantingTempDir) fullNameFor: 'mediaList.dat').

			fStream setToEnd.
			fStream
				nextPutAll: (anExporter sprite filenameForSound: english); cr.
			fStream close.
			
			"Now export this filename to the method the NXT will call to play the sound."
			^ anExporter codeStream nextPutAll: '"', (anExporter sprite filenameForSound: english), '"'
		].

	(type = $a) ifTrue:
		[
			"Exporting an attribute, in the '%a of %m' block"
			"As the terms already exist, we need to make them different."
	 		^ anExporter outputTranslationOfTerm: (english, ' of').
		].

	"Everything else should be translated; for example, 'flute' might become 'INSTRUMENT_FLUTE'"
	 ^ anExporter outputTranslationOfTerm: english.
	
	! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'jm 8/6/2008 20:54'!
getOptionsSelector

	^ getOptionsSelector
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'jm 5/14/2004 11:50'!
getOptionsSelector: aSymbolOrString
	"Set a selector that is used to get a set of options from my command block's receiver. This is used when the options can change over time, such as selecting from a list of available sounds or costumes in the objects media library."

	getOptionsSelector _ aSymbolOrString asSymbol.
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'CWB 12/17/2010 11:22'!
indexOfChoice
"Returns a number indicating where, in a list of options, the given choice is (or nil)"

	^ self options indexOf: (self choice) ifAbsent: [ nil ].! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'CWB 12/15/2010 21:16'!
options
	"Answer my set of options. If getOptionsSelector is not nil and I'm in a CommandBlock with a non-nil receiver, fetch my set of options by sending getOptionsSelector to that receiver."

	#varNamesMenu = getOptionsSelector
		ifTrue: [^owner varNamesMenu].

	(getOptionsSelector notNil and:
	 [(owner respondsTo: #receiver) and:
	 [owner receiver notNil]]) ifTrue: [
		^ owner receiver perform: getOptionsSelector].

	^ options
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'jm 8/24/2003 18:32'!
options: stringList

	options _ stringList asArray.
	(options includes: choice) ifFalse: [self choice: options first].
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'CWB 11/24/2012 16:09'!
presentMenu
	"Pop up a menu of choices."

	| optionList doLocalization menu selection option |
	optionList _ self options.
	optionList size = 0 ifTrue: [^ self].
	doLocalization _ (#(costumeNames soundNames varNamesMenu listVarMenu applicablePeripheralNames) includes: getOptionsSelector) not.
	doLocalization ifFalse: [
		doLocalization = (optionList = {'with inputs' asUTF8 localized. 'with input list' asUTF8 localized})].
	menu _ CustomMenu new.
	optionList do: [:sym |
		'-' = sym
			ifTrue: [menu addLine]
			ifFalse: [
				doLocalization & sym isUnicode not
					ifTrue: [option _ sym localized asUTF8]
					ifFalse: [option _ sym].
				((#(hookupSensorNames hookupBooleanSensorNames) includes: getOptionsSelector) and:
				 [(#(connected resistance) includes: sym)])
					ifTrue: [#(A B C D) do: [:c |
						self owner isBooleanReporter
							ifFalse: [menu add: (option, '-', c) action: (sym, '-', c)]
							ifTrue: [menu add: (c asUTF8, ' ', option) action: (c, ' ' ,sym)]]]
					ifFalse: [
						"Typically we have a text string."
						"But there is a special case where we use an image instead."

						| imageForm imageName |

						(option beginsWith: 'img:') ifTrue: [
							imageName _ (option allButFirst: 4), 'Black'.
							imageForm _ ScratchFrameMorph 
								skinAt: (imageName asSymbol) 
								ifAbsent: [ imageForm _ nil ].
						].
				
						imageForm ifNotNil: [
							menu addIcon: imageForm toolTip: nil action: sym.
						] ifNil: [
							"Here is where we normally add something to the list"
							menu add: option action: sym.
						].
					]
			]
	].

	(selection _ menu startUp) ifNotNil: [self choice: selection].
! !

!ChoiceArgMorph methodsFor: 'accessing' stamp: 'jens 2/19/2010 01:55'!
privateSetChoice: aSymbol
	"Update the toggle button when an argument is changed within a block."

	| frame palette block doLocalization label |
	frame _ self ownerThatIsA: ScratchFrameMorph.
	((#soundNames = getOptionsSelector) and:
	 [aSymbol = ('record' localized, ScratchTranslator ellipsesSuffix)]) ifTrue: [
		frame ifNotNil: [^ frame newSound]].

	((#sceneNames = getOptionsSelector) and:
	 [aSymbol = ('record' localized, ScratchTranslator ellipsesSuffix)]) ifTrue: [
		frame ifNotNil: [^ frame workPane newScene]].

	((#sceneNames = getOptionsSelector) and:
	 [aSymbol = ('re-record' localized, ScratchTranslator ellipsesSuffix)]) ifTrue: [
		frame ifNotNil: [^ frame workPane rerecordScene]].

	choice _ label _ aSymbol.
	label isUnicode ifFalse: [label _ label asString].
	doLocalization _ (#(costumeNames soundNames varNamesMenu listVarMenu) includes: getOptionsSelector) not.
	doLocalization & label isUnicode not
		ifTrue:[labelMorph contents: label localized]
		ifFalse:[labelMorph contents: label].
	labelMorph position: self position + borderWidth + (3 @ -1).
	self extent: (labelMorph extent + (16@2) + borderWidth).

	menuIconMorph position: (self right - borderWidth - 10)@(self top + borderWidth + 4).

	(block _ self ownerThatIsA: HatBlockMorph) ifNotNil: [
		block fixBlockLayout].

	(owner isKindOf: SetterBlockMorph) ifTrue: [ owner determineIfSpecialForm ].

"
	options = #('with inputs' 'with input list') ifTrue: [
		(owner respondsTo: #adjustInputParadigm:) ifTrue: [
			owner adjustInputParadigm: choice ]].
"
	(palette _ self ownerThatIsA: ScratchBlockPaletteMorph) ifNil: [^ self].
	(block _ self ownerThatIsA: BlockMorph) ifNil: [^ self].

	block isReporter ifTrue: [palette updateWatcherButtonsForFrame: frame].
! !


!ChoiceArgMorph methodsFor: 'event handling' stamp: 'jm 1/23/2007 12:32'!
handlesMouseDown: evt

	^ evt hand toolType isNil and:
		[(self topRight - (14@0) corner: self bottomRight) containsPoint: evt cursorPoint]
! !

!ChoiceArgMorph methodsFor: 'event handling' stamp: 'jm 12/13/2006 21:35'!
mouseDown: evt

	self presentMenu.
! !

!ChoiceArgMorph methodsFor: 'event handling' stamp: 'jm 12/14/2004 11:51'!
wantsKeyboardFocusFor: aSubmorph
	"Don't allow shift-click editing of block labels."

	^ false
! !


!ChoiceArgMorph methodsFor: 'other' stamp: 'jens 2/4/2010 00:06'!
evaluate
	"Answer my value."

	(self isEmptySlot and: [implicitValue notNil])
		ifTrue: [^implicitValue].

	isBoolean ifTrue: [^ 'true' = choice].
	^ choice
! !

!ChoiceArgMorph methodsFor: 'other' stamp: 'jm 4/1/2005 20:33'!
fixArgLayout
	"Set my color to my owner's color."

	self color: (owner color mixed: 0.75 with: (Color gray: 0.45)).
! !

!ChoiceArgMorph methodsFor: 'other' stamp: 'jm 10/30/2005 08:32'!
printArgOn: aStream
	"Print this argument morph on the given stream."

	aStream nextPutAll: '"', self labelMorph contents, '"'.
! !


!ChoiceArgMorph methodsFor: 'object i/o' stamp: 'jm 5/14/2004 11:39'!
fieldsVersion

	^ 2
! !

!ChoiceArgMorph methodsFor: 'object i/o' stamp: 'jm 5/14/2004 11:54'!
initFieldsFrom: anObjStream version: classVersion

	super initFieldsFrom: anObjStream version: classVersion.
	self initFieldsNamed: #(
		isBoolean
		options
		choice
	) from: anObjStream.
	classVersion = 1 ifTrue: [^ self].

	"fields added in version 2"
	self initFieldsNamed: #(
		getOptionsSelector
	) from: anObjStream.
! !

!ChoiceArgMorph methodsFor: 'object i/o' stamp: 'jm 5/14/2004 11:54'!
storeFieldsOn: anObjStream

	super storeFieldsOn: anObjStream.
	self storeFieldsNamed: #(
		isBoolean
		options
		choice
		getOptionsSelector
	) on: anObjStream.
! !


!ChoiceArgMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/22/2011 23:43'!
acceptsDroppedReporters
	"Accept reporter blocks in list blocks"

	(owner respondsTo: #selector) ifFalse: [
		^ false].

	^ #(
		listNamed: 
		append:toList: 
		deleteLine:ofList: 
		insert:at:ofList: 
		setLine:ofList:to: 
		getLine:ofList: 
		lineCountOfList: 
		list:contains:
		contentsOfList:
		
		getAttribute:of:
		attribute:of:
		get:
		set:to:
		setVar:to:

"		doRun
		doRunBlockWithArgs
		doFork
		doForkBlockWithArgs
		doReport
		doCallBlockWithArgs
"
	) includes: owner selector
! !

!ChoiceArgMorph methodsFor: 'as yet unclassified' stamp: 'jens 11/11/2009 22:44'!
acceptsTypeOf: aBlockMorph
	"Answer true if I can accept a dropped reporter of the given type."

	aBlockMorph isReporter ifFalse: [^ false].
	^ aBlockMorph isBooleanReporter not  "must be a number"
! !


!ChoiceArgMorph methodsFor: 'BYOB' stamp: 'jens 2/3/2010 23:32'!
isEmptySlot
	^choice size = 0! !
